@@ -31,7 +31,9 @@
 
 */
 
-#include <uim/config.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
 #include <gtk/gtk.h>
@@ -40,70 +42,38 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define NR_CANDIDATES 10 /* this should be configurable*/
+#define NR_CANDIDATES 10 /* This should be configurable */
 
 typedef struct {
-  GtkWidget *top_win;
-  GtkWidget *clist;
+  GtkWidget *window;
+  GtkWidget* view;
   GtkWidget *numlabel;
+  GtkListStore* store;
   int nr_candidates;
   int candidate_index;
   int is_active;
-  int layout_begin;
-  int select_count;
-  GdkRectangle cursor;
 }candidate_window;
 
 static candidate_window cwin;
 static int read_tag;
-static GtkWidget* m_window;
-static GtkWidget* m_view;
-static GtkListStore* m_store;   
-
-void
-select_candidate_cb(GtkWidget *clist,
-		    gint row, gint column,
-		    GdkEventButton *event,
-		    gpointer data)
-{
-  if (cwin.select_count == 0) {
-    int idx = row + cwin.layout_begin;
-    
-    if( idx >= cwin.nr_candidates ) {
-      gtk_clist_select_row(GTK_CLIST(cwin.clist),
-			   cwin.candidate_index - cwin.layout_begin,
-			   0);
-      return;
-    }
-    
-    //    uim_set_candidate_index(ic->mUc, idx);
-    cwin.candidate_index = idx;
-  } else {
-    cwin.select_count --;
-  }
-}
 
 void
 init_candidate_win(void)
 {
-  GtkListStore* store;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column; 
   
-  GtkWidget* window;
-  GtkWidget* scrw;
-  GtkWidget* tree_view;
-  
+  GtkWidget* scrw;  
   GtkWidget *vbox;
   GtkWidget *numlabel;
 
   vbox = gtk_vbox_new(FALSE, 0);
 
-  store = gtk_list_store_new(2, G_TYPE_UINT, G_TYPE_STRING);
+  cwin.store = gtk_list_store_new(2, G_TYPE_UINT, G_TYPE_STRING);
   
-  window = gtk_window_new(GTK_WINDOW_POPUP);
-  gtk_container_set_border_width(GTK_CONTAINER(window), 1);
-  gtk_window_set_default_size(GTK_WINDOW(window), 80, 150);
+  cwin.window = gtk_window_new(GTK_WINDOW_POPUP);
+  gtk_container_set_border_width(GTK_CONTAINER(cwin.window), 1);
+  gtk_window_set_default_size(GTK_WINDOW(cwin.window), 80, 150);
   
   scrw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw),
@@ -111,12 +81,12 @@ init_candidate_win(void)
 				 GTK_POLICY_AUTOMATIC);
   gtk_box_pack_start(GTK_BOX(vbox), scrw, TRUE, TRUE, 0);
   
-  tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree_view), TRUE);
-  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view), FALSE);
-  gtk_container_add(GTK_CONTAINER(scrw), tree_view);
+  cwin.view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(cwin.store));
+  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(cwin.view), TRUE);
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(cwin.view), FALSE);
+  gtk_container_add(GTK_CONTAINER(scrw), cwin.view);
 
-  gtk_container_add(GTK_CONTAINER(window), vbox);
+  gtk_container_add(GTK_CONTAINER(cwin.window), vbox);
     
   renderer = gtk_cell_renderer_text_new();
   column = gtk_tree_view_column_new_with_attributes("No",
@@ -124,7 +94,7 @@ init_candidate_win(void)
 						    "text",
 						    0,
 						    NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(cwin.view), column);
   
   renderer = gtk_cell_renderer_text_new();
   column = gtk_tree_view_column_new_with_attributes("Text",
@@ -132,22 +102,16 @@ init_candidate_win(void)
 						    "text",
 						    1,
 						    NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(cwin.view), column);
   
   numlabel = gtk_label_new("");
 
   gtk_box_pack_start(GTK_BOX(vbox), numlabel, FALSE, FALSE, 0);
 
-  m_window = window;
-  m_store = store;
-  m_view = tree_view;
   cwin.numlabel = numlabel;
 
   cwin.is_active = 0;
 
-  /*  g_signal_connect(G_OBJECT(cwin.clist), "select_row",
-		   G_CALLBACK(select_candidate_cb),
-		   NULL);*/
   gtk_widget_show(numlabel);
   gtk_widget_show_all(vbox);
 }
@@ -155,14 +119,12 @@ init_candidate_win(void)
 void
 candwin_activate(char **str)
 {
-  GtkListStore* store;
   gsize rbytes, wbytes;
   GtkTreeIter ti;
   int i,j = 1;
   char *utf8_str;
 
-  store = m_store;
-  gtk_list_store_clear(store);
+  gtk_list_store_clear(cwin.store);
   
   for(i = 2; str[i];i++) {
 
@@ -175,8 +137,8 @@ candwin_activate(char **str)
 			 "EUC-JP",
 			 &rbytes, &wbytes, NULL);
     
-    gtk_list_store_append(store, &ti);
-    gtk_list_store_set(store, &ti,
+    gtk_list_store_append(cwin.store, &ti);
+    gtk_list_store_set(cwin.store, &ti,
 		       0, j,
 		       1, utf8_str,
 		       -1);
@@ -184,7 +146,7 @@ candwin_activate(char **str)
     g_free(utf8_str);
   }
   
-  gtk_widget_show_all(m_window);
+  gtk_widget_show_all(cwin.window);
   cwin.nr_candidates = i - 2 ;
   cwin.is_active = 1;
 }
@@ -200,9 +162,9 @@ candwin_update(char **str)
   cwin.candidate_index = index;
 
   path = gtk_tree_path_new_from_indices(index, -1);
-  gtk_tree_view_set_cursor(GTK_TREE_VIEW(m_view),
+  gtk_tree_view_set_cursor(GTK_TREE_VIEW(cwin.view),
 			   path, NULL, FALSE);
-  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(m_view),
+  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(cwin.view),
 			       path, NULL, FALSE, 0.0, 0.0);
 
   sprintf(idx, "%d / %d", cwin.candidate_index + 1 , cwin.nr_candidates); 
@@ -217,8 +179,7 @@ candwin_move(char **str)
   int x,y;
   sscanf(str[1],"%d",&x);
   sscanf(str[2],"%d",&y);
-  gtk_window_move(GTK_WINDOW(m_window), x, y );
-  //  layout_candidate();
+  gtk_window_move(GTK_WINDOW(cwin.window), x, y );
 }
 
 
@@ -226,13 +187,13 @@ void
 candwin_show(void)
 {
   if(cwin.is_active)
-    gtk_widget_show_all(m_window);
+    gtk_widget_show_all(cwin.window);
 }
 
 void
 candwin_deactivate(void)
 {
-  gtk_widget_hide(m_window);
+  gtk_widget_hide(cwin.window);
   cwin.is_active = 0;
 }
 
@@ -252,7 +213,7 @@ void str_parse(char *str)
     } else if(strcmp("show", tmp[i]) == 0) {
       candwin_show();
     } else if(strcmp("hide", tmp[i]) == 0) {
-      gtk_widget_hide_all(m_window);
+      gtk_widget_hide_all(cwin.window);
     } else if(strcmp("move", tmp[i]) == 0) {
       candwin_move(tmp);
     } else if(strcmp("deactivate", tmp[i]) == 0) {
@@ -281,7 +242,8 @@ read_cb(gpointer p, int fd, GdkInputCondition c)
   exit(-1);
 }
 
-int main (int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
   gtk_set_locale();
   gtk_init( &argc, &argv );
