@@ -33,7 +33,7 @@
 (require "japanese.scm")
 (require "generic-key.scm")
 ;;
-(define anthy-init-lib-ok? '())
+(define anthy-init-lib-ok? #f)
 ;; configs
 (define anthy-use-candidate-window? #t)
 (define anthy-candidate-op-count 1)
@@ -47,9 +47,13 @@
 (define-key anthy-wide-latin-key? "L")
 (define-key anthy-hankaku-kana-key? '("<Control>q" "<Control>Q"))
 (define-key anthy-kana-toggle-key? "q")
-(define-key anthy-commit-as-opposite-kana-key? "Q")
-(define-key anthy-begin-conv-key? " ")
 (define-key anthy-commit-key? 'generic-commit-key?)
+(define-key anthy-commit-as-opposite-kana-key? "Q")
+(define-key anthy-commit-as-katakana-key? "F7")
+(define-key anthy-commit-as-hankana-key? "F8")
+(define-key anthy-commit-as-wide-latin-key? "") ; "F9"
+(define-key anthy-commit-as-latin-key? "") ; "F10"
+(define-key anthy-begin-conv-key? 'generic-begin-conv-key?)
 (define-key anthy-extend-segment-key? '("<Control>o" "<Control>O" "<Shift>right"))
 (define-key anthy-shrink-segment-key? '("<Control>i" "<Control>I" "<Shift>left"))
 (define-key anthy-next-candidate-key? 'generic-next-candidate-key?)
@@ -189,11 +193,9 @@
  (lambda ()
    (let ((c '())
 	 (rkc (rk-context-new ja-rk-rule #t #f)))
-     (set! c (copy-list '(() () () () () () () () () () () 0 #t #f)))
-     (if (and
-	  (not anthy-init-lib-ok?)
-	  (symbol-bound? 'anthy-lib-init))
-	 (set! anthy-init-lib-ok? (anthy-lib-init)))
+     (set! c (copy-list '(() () () () () () () () () () #f 0 #t #f)))
+     (if (symbol-bound? 'anthy-lib-init))
+	 (set! anthy-init-lib-ok? (anthy-lib-init))
      (anthy-context-set-ac-id!
       c
       (if anthy-init-lib-ok?
@@ -209,54 +211,34 @@
     (im-commit-raw id)
     (anthy-context-set-commit-raw! ac #t)))
 
-(define anthy-opposite-kana
-  (lambda (kana)
-    (cond
-     ((= kana anthy-type-hiragana)
-      anthy-type-katakana)
-     ((= kana anthy-type-katakana)
-      anthy-type-hiragana)
-     ((= kana anthy-type-hankana)
-      anthy-type-hiragana))))
-
 (define anthy-context-kana-toggle
   (lambda (ac)
     (let* ((kana (anthy-context-kana-mode ac))
-	   (opposite-kana (anthy-opposite-kana kana)))
+	   (opposite-kana (multi-segment-opposite-kana kana)))
       (anthy-context-set-kana-mode! ac opposite-kana))))
 
-(define anthy-make-string
-  (lambda (sl dir type)
-    (let ((get-str-by-type 
-	   (lambda (l)
-	     (cond
-	      ((= type anthy-type-hiragana)
-	       (caar l))
-	      ((= type anthy-type-katakana)
-	       (car (cdar l)))
-	      ((= type anthy-type-hankana)
-	       (cadr (cdar l)))))))
-      (if sl
-	  (if dir
-	      (string-append (anthy-make-string (cdr sl) dir type)
-			     (get-str-by-type sl))
-	      (string-append (get-str-by-type sl)
-			     (anthy-make-string (cdr sl) dir type)))
-	  ""))))
-
-(define anthy-make-left-string
-  (lambda (sl kana)
-    (anthy-make-string sl #t kana)))
-
-(define anthy-make-right-string
-  (lambda (sl kana)
-    (anthy-make-string sl #f kana)))
-
-;(define anthy-get-candidate
-;  (lambda (ac nth-seg)
-;    (let ((id (anthy-context-ac-id ac))
-;	  (idx (nth (anthy-context-index-list ac) nth-seg)))
-;      ())))
+;; TODO: generarize as multi-segment procedure
+;; side effect: flushes rkc if convert-pending?
+(define anthy-make-whole-string
+  (lambda (ac convert-pending-into-kana? kana)
+    (let* ((rkc (anthy-context-rkc ac))
+	   (pending (rk-pending rkc))
+	   (residual-kana (rk-push-key-last! rkc)))
+      (string-append
+       (multi-segment-make-left-string (anthy-context-left-string ac)
+				       kana)
+       (if convert-pending-into-kana?
+	   (if residual-kana
+	       (begin
+		 (rk-flush rkc)
+		 (multi-segment-make-left-string (list residual-kana)
+						 kana))
+	       (if using-kana-table?
+		   pending
+		   ""))
+	   pending)
+       (multi-segment-make-right-string (anthy-context-right-string ac)
+					kana)))))
 
 (define anthy-init-handler
   (lambda (id arg)
@@ -361,22 +343,10 @@
   (lambda (ac)
     (let* ((ac-id (anthy-context-ac-id ac))
 	   (rkc (anthy-context-rkc ac))
-	   (pending (rk-pending rkc))
-	   (kana (anthy-context-kana-mode ac))
-	   (last "")
-	   (res (rk-push-key-last! rkc)))
-      ;; get residual 'n'
-      (if res
-	  (begin
-	    (anthy-append-string ac res)
-	    (set! pending "")))
-      ;;
+	   (kana (anthy-context-kana-mode ac)))
       (anthy-lib-set-string
        ac-id
-       (string-append
-	(anthy-make-left-string (anthy-context-left-string ac)  anthy-type-hiragana)
-	pending
-	(anthy-make-right-string (anthy-context-right-string ac) anthy-type-hiragana)))
+       (anthy-make-whole-string ac #t anthy-type-hiragana))
       (anthy-context-set-index-list!
        ac
        (multi-segment-make-index-list
@@ -434,16 +404,19 @@
        (direct
 	(im-commit id direct))
 
+       ((symbol? key)
+	(anthy-commit-raw ac id))
+
        (else
-	(begin
-	  (set! key-str (charcode->string (to-lower-char key)))
-	  (string-find (rk-expect rkc) key-str)
-	  (let ((res (rk-push-key! rkc key-str)))
-	    (if res
-		(anthy-append-string ac res)
-		(if (not (rk-pending rkc)
-			 (anthy-commit-raw ac id)))))))
-       ))))
+	(let* ((key-str (charcode->string
+			 (if using-kana-table?
+			     key
+			     (to-lower-char key))))
+	       (res (rk-push-key! rkc key-str)))
+	  (if res
+	      (anthy-append-string ac res)
+	      (if (not (rk-pending rkc)
+		       (anthy-commit-raw ac id))))))))))
 
 (define anthy-has-preedit?
   (lambda (ac)
@@ -488,17 +461,45 @@
 	(begin
 	  (rk-flush rkc)
 	  (anthy-context-set-left-string! ac ())))
-       
-       ;; Ҥ餬ʥ⡼ɤǥʤꤹ
+
+       ;; ߤȤϵդΤʥ⡼ɤǤʤꤹ
        ((anthy-commit-as-opposite-kana-key? key key-state)
 	(begin
 	  (im-commit
 	   id
-	   (string-append
-	    (anthy-make-left-string (anthy-context-left-string ac)
-				    (anthy-opposite-kana kana))
-	    (anthy-make-right-string (anthy-context-right-string ac)
-				     (anthy-opposite-kana kana))))
+	   (anthy-make-whole-string ac #t (multi-segment-opposite-kana kana)))
+	  (anthy-flush ac)))
+
+       ;; ʥ⡼ɤǤʤꤹ
+       ((anthy-commit-as-katakana-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   (anthy-make-whole-string ac #t multi-segment-type-katakana))
+	  (anthy-flush ac)))
+
+       ;; Ⱦѥʥ⡼ɤǤʤꤹ
+       ((anthy-commit-as-hankana-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   (anthy-make-whole-string ac #t multi-segment-type-hankana))
+	  (anthy-flush ac)))
+
+       ;; ʤѿᤷƳꤹ
+       ((anthy-commit-as-latin-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   "not implemented")
+	  (anthy-flush ac)))
+
+       ;; ʤѱѿᤷƳꤹ
+       ((anthy-commit-as-wide-latin-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   "")
 	  (anthy-flush ac)))
 
        ;; ߤΤʤ塢Ҥ餬/ʥ⡼ɤڤ괹
@@ -506,11 +507,7 @@
 	(begin
 	  (im-commit
 	   id
-	   (string-append
-	    (anthy-make-left-string (anthy-context-left-string ac)
-				    kana)
-	    (anthy-make-right-string (anthy-context-right-string ac)
-				     kana)))
+	   (anthy-make-whole-string ac #t kana))
 	  (anthy-flush ac)
 	  (anthy-context-kana-toggle ac)
 	  (anthy-update-mode ac id)
@@ -525,10 +522,7 @@
 	(begin
 	  (im-commit
 	   id
-	   (string-append
-	    (anthy-make-left-string (anthy-context-left-string ac) kana)
-	    (rk-pending rkc)
-	    (anthy-make-right-string (anthy-context-right-string ac) kana)))
+	   (anthy-make-whole-string ac #t kana))
 	  (anthy-flush ac)))
 
        ;; left
@@ -578,14 +572,16 @@
 	(anthy-commit-raw ac id))
 
        (else	
-	(let* ((key-str (charcode->string key))
+	(let* ((key-str (charcode->string 
+			 (if using-kana-table?
+			     key
+			     (to-lower-char key))))
 	       (pend (rk-pending rkc))
 	       (res (rk-push-key! rkc key-str)))
-	  (if (and res (not (string-equal? (car res) "")))
+	  (if (and res (not (string=? (car res) "")))
 	      (anthy-append-string ac res)
 	      (if using-kana-table?
-		  (anthy-append-string ac (list pend "" "")))
-	      )))))))
+		  (anthy-append-string ac (list pend "" ""))))))))))
 
 (define anthy-proc-input-state
   (lambda (ac id key key-state)
@@ -631,14 +627,14 @@
       (im-clear-preedit id)
       (im-pushback-preedit
        id preedit-underline
-       (anthy-make-left-string (anthy-context-left-string ac) kana))
+       (multi-segment-make-left-string (anthy-context-left-string ac) kana))
       (im-pushback-preedit id preedit-underline
                            (rk-pending rkc))
       (if (anthy-has-preedit? ac)
 	  (im-pushback-preedit id preedit-cursor ""))
       (im-pushback-preedit
        id preedit-underline
-       (anthy-make-right-string (anthy-context-right-string ac) kana))
+       (multi-segment-make-right-string (anthy-context-right-string ac) kana))
       (im-update-preedit id))))
   
 (define anthy-get-commit-string
@@ -735,6 +731,7 @@
 	 (anthy-context-cur-seg ac))))
       )))
 
+;; 'compose' means 'converting' here
 (define anthy-proc-compose-state
   (lambda (ac id key key-state)
     (cond
@@ -789,14 +786,20 @@
      ((anthy-next-page-key? key key-state)
 	(im-shift-page-candidate id #t))
 
-     ((modifier-key-mask key-state)
-      ())
+     ;; don't discard shift-modified keys. Some of them ("?", "~",
+     ;; etc) are used to implicit commit. Reported by [Anthy-dev 745]
+     ;; -- YamaKen 2004-04-08
+     ((and (modifier-key-mask key-state)
+	   (not (shift-key-mask key-state)))
+      #f)  ;; use #f rather than () to conform to R5RS
+
+     ((symbol? key)
+      #f)
 
      (else
       (begin
 	(anthy-do-commit ac id)
-	(anthy-proc-input-state ac id key key-state))))
-    ))
+	(anthy-proc-input-state ac id key key-state))))))
 
 (define anthy-proc-wide-latin
   (lambda (c key key-state)
@@ -878,32 +881,32 @@
       (anthy-flush ac)
       (anthy-update-preedit ac id)
       (cond
-       ((string-equal? message
+       ((string=? message
 		       "prop_anthy_hiragana")
 	(begin
 	  (anthy-context-set-on! ac #t)
 	  (anthy-context-set-kana-mode! ac anthy-type-hiragana)))
-       ((string-equal? message
+       ((string=? message
 		       "prop_anthy_katakana")
 	(begin
 	  (anthy-context-set-on! ac #t)
 	  (anthy-context-set-kana-mode! ac anthy-type-katakana)))
-       ((string-equal? message
+       ((string=? message
 		       "prop_anthy_hankana")
 	(begin
 	  (anthy-context-set-on! ac #t)
 	  (anthy-context-set-kana-mode! ac anthy-type-hankana)))
-       ((string-equal? message
+       ((string=? message
 		       "prop_anthy_direct")
 	(begin
 	  (anthy-context-set-on! ac #f)
 	  (anthy-context-set-wide-latin! ac #f)))
-       ((string-equal? message
+       ((string=? message
 		       "prop_anthy_zenkaku")
 	(begin
 	  (anthy-context-set-on! ac #f)
 	  (anthy-context-set-wide-latin! ac #t)))
-       ((string-equal? message
+       ((string=? message
 		       "prop_anthy_kana")
 	(begin
 	  (load-kana-table))))
@@ -973,7 +976,7 @@
  'anthy
  "ja"
  "EUC-JP"
- '()
+ #f
  anthy-init-handler
  anthy-release-handler
  anthy-mode-handler
