@@ -100,6 +100,12 @@
     ((("k" "y" "e"). ())("" "" ""))
     ((("k" "y" "o"). ())("" "" ""))
 
+    ((("k" "w" "a"). ())("" "" ""))
+    ((("k" "w" "i"). ())("" "" ""))
+    ((("k" "w" "u"). ())("" "" ""))
+    ((("k" "w" "e"). ())("" "" ""))
+    ((("k" "w" "o"). ())("" "" ""))
+ 
     ((("g" "g"). ("g"))("" "" ""))
 
 
@@ -115,6 +121,12 @@
     ((("g" "y" "e"). ())("" "" "ގ"))
     ((("g" "y" "o"). ())("" "" "ގ"))
 
+    ((("g" "w" "a"). ())("" "" "ގ"))
+    ((("g" "w" "i"). ())("" "" "ގ")) 
+    ((("g" "w" "u"). ())("" "" "ގ")) 
+    ((("g" "w" "e"). ())("" "" "ގ")) 
+    ((("g" "w" "o"). ())("" "" "ގ"))
+
     ((("s" "s"). ("s"))("" "" ""))
 
     ((("s" "s"). ())("s" "s" "s"))
@@ -135,6 +147,12 @@
     ((("s" "h" "u"). ())("" "" ""))
     ((("s" "h" "e"). ())("" "" ""))
     ((("s" "h" "o"). ())("" "" ""))
+
+    ((("s" "w" "a"). ())("" "" ""))
+    ((("s" "w" "i"). ())("" "" ""))
+    ((("s" "w" "u"). ())("" "" ""))
+    ((("s" "w" "e"). ())("" "" ""))
+    ((("s" "w" "o"). ())("" "" "")) 
     
     ((("z" "z"). ("z"))("" "" ""))
 
@@ -177,20 +195,31 @@
     ((("t" "y" "u"). ())("" "" ""))
     ((("t" "y" "e"). ())("" "" ""))
     ((("t" "y" "o"). ())("" "" ""))
+
+    ((("t" "w" "a"). ())("Ȥ" "ȥ" "Ď"))
+    ((("t" "w" "i"). ())("Ȥ" "ȥ" "Ď"))
     ((("t" "w" "u"). ())("Ȥ" "ȥ" "Ď"))
+    ((("t" "w" "e"). ())("Ȥ" "ȥ" "Ď"))
+    ((("t" "w" "o"). ())("Ȥ" "ȥ" "Ď"))
+
     ((("t" "h" "a"). ())("Ƥ" "ƥ" "Î"))
     ((("t" "h" "i"). ())("Ƥ" "ƥ" "Î"))
     ((("t" "h" "u"). ())("Ƥ" "ƥ" "Î"))
     ((("t" "h" "e"). ())("Ƥ" "ƥ" "Î"))
     ((("t" "h" "o"). ())("Ƥ" "ƥ" "Î"))
     
+    ((("t" "s" "a"). ())("Ĥ" "ĥ" ""))
+    ((("t" "s" "i"). ())("Ĥ" "ĥ" ""))
+    ((("t" "s" "u"). ())(""   ""   ""))
+    ((("t" "s" "e"). ())("Ĥ" "ĥ" ""))
+    ((("t" "s" "o"). ())("Ĥ" "ĥ" ""))    
+
     ((("c" "h" "a"). ())("" "" ""))
     ((("c" "h" "i"). ())("" "" ""))
     ((("c" "h" "u"). ())("" "" ""))
     ((("c" "h" "e"). ())("" "" ""))
     ((("c" "h" "o"). ())("" "" ""))
     
-    ((("t" "s" "u"). ())("" ""))
     ((("x" "t" "u"). ())("" "" ""))
     ((("x" "t" "s" "u"). ())("" "" ""))
     ((("c" "c"). ("c"))("" "" ""))
@@ -215,9 +244,16 @@
     ((("d" "h" "e"). ())("Ǥ" "ǥ" "Îގ"))
     ((("d" "h" "o"). ())("Ǥ" "ǥ" "Îގ"))
 
+
+    ((("d" "w" "a"). ())("ɤ" "ɥ" " ގ"))
+    ((("d" "w" "i"). ())("ɤ" "ɥ" " ގ"))
+    ((("d" "w" "u"). ())("ɤ" "ɥ" "Ďގ"))
+    ((("d" "w" "e"). ())("ɤ" "ɥ" " ގ"))
+    ((("d" "w" "o"). ())("ɤ" "ɥ" " ގ"))
+
     ((("n" "n"). ())("" "" ""))
     ((("n"). ())("" "" ""))
-    ((("n" "n"). ())("" "" ""))
+    ((("n" ""). ())("" "" ""))
 
     ((("n" "a"). ())("" "" ""))
     ((("n" "i"). ())("" "" ""))
@@ -259,6 +295,8 @@
     ((("f" "y" "e"). ())("դ" "ե" "̎"))
     ((("f" "y" "o"). ())("դ" "ե" "̎"))
 
+    ((("f" "w" "u"). ())("դ" "ե" "   ")) 
+
     ((("b" "b"). ("b"))("" "" ""))
 
     ((("b" "a"). ())("" "" "ʎ"))
@@ -308,8 +346,12 @@
 
     ((("y" "a"). ())("" "" ""))
     ((("y" "u"). ())("" "" ""))
+    ((("y" "e"). ())("" "" ""))
     ((("y" "o"). ())("" "" ""))
-    
+   
+    ((("x" "k" "a"). ())("" "" ""))
+    ((("x" "k" "e"). ())("" "" ""))
+ 
     ((("x" "y" "a"). ())("" "" ""))
     ((("x" "y" "u"). ())("" "" ""))
     ((("x" "y" "o"). ())("" "" ""))
@@ -338,6 +380,8 @@
     ((("w" "e"). ())("" "" ""))
     ((("x" "w" "e"). ())("" "" ""))
     ((("w" "o"). ())("" "" ""))
+    ((("w" "h" "a"). ())("" "" "")) 
+    ((("w" "h" "o"). ())("" "" "")) 
 
     ((("v" "v"). ("v"))("" "" ""))
 
@@ -347,6 +391,10 @@
     ((("v" "e"). ())("" "" "ގ"))
     ((("v" "o"). ())("" "" "ގ"))
 
+    ((("v" "y" "a"). ())("" "" "ގ")) 
+    ((("v" "y" "u"). ())("" "" "ގ")) 
+    ((("v" "y" "o"). ())("" "" "ގ")) 
+
     ((("z" "k"). ())("" ""))
     ((("z" "j"). ())("" ""))
     ((("z" "h"). ())("" ""))
@@ -476,7 +524,7 @@
     (if (null? rule)
 	#f
 	(let ((r (car rule)))
-	  (if (string-equal? c (car r))
+	  (if (string=? c (car r))
 	      (cadr r)
 	      (ja-find-rec c (cdr rule)))))))
 
