@@ -28,6 +28,8 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
+(require "util.scm")
+
 ;
 (define shift-key-mask
   (lambda (state)
@@ -107,7 +109,7 @@
 
 (define parse-tag-prefix-symbol
   (lambda (parsed char-list)
-    (let ((prefix (if (string-equal? parsed "")
+    (let ((prefix (if (string=? parsed "")
 		      #f
 		      (intern-key-prefix parsed tag-prefix-alist))))
       (if (not (null? char-list))
@@ -115,23 +117,23 @@
 		 (head-char (string->charcode head))
 		 (rest (cdr char-list)))
 	    (if (or (alphabet-char? head-char)
-		    (numeral? head-char)
-		    (string-equal? head "_"))
+		    (numeral-char? head-char)
+		    (string=? head "_"))
 		(parse-tag-prefix-symbol (string-append parsed head) rest)
 		(cons prefix char-list)))
 	  (cons prefix ())))))
 
 (define parse-tag-prefix
   (lambda (str)
-    (if (not (string-equal? str ""))
+    (if (not (string=? str ""))
 	(let* ((char-list (reverse (string-to-list str)))
 	       (head (car char-list)))
-	  (if (string-equal? head "<")
+	  (if (string=? head "<")
 	      (let* ((parsed (parse-tag-prefix-symbol "" (cdr char-list)))
 		     (prefix (car parsed))
 		     (rest (cdr parsed)))
 		(if (and (not (null? rest))
-			 (string-equal? (car rest) ">"))
+			 (string=? (car rest) ">"))
 		    (cons prefix
 			  (if (null? (cdr rest))
 			      ""
@@ -144,7 +146,7 @@
   (lambda (str)
     (let* ((char-list (reverse (string-to-list str)))
 	   (prefix-str (and (<= 2 (length char-list))
-			(string-equal? (nth 1 char-list) "-")
+			(string=? (nth 1 char-list) "-")
 			(car char-list)))
 	   (prefix (intern-key-prefix prefix-str emacs-like-prefix-alist))
 	   (rest (if prefix
@@ -200,7 +202,7 @@
 				      translators)))
 	      (parse-key-str rest translators key key-state)))
 	   (else
-	    (let ((key (intern str))	; keysym
+	    (let ((key (string->symbol str))	; keysym
 		  (rest ""))
 	      (parse-key-str rest translators key key-state))))))))))
 
@@ -218,6 +220,8 @@
 	   translated-key
 	   translated-state)))))
 
+;; Generates key predicate
+;; (make-single-key-predicate "<Control>j")
 (define make-single-key-predicate
   (lambda (source)
     (cond
@@ -242,16 +246,18 @@
       (let ((maybe-predicate source))
 	maybe-predicate)))))
 
+;; Generates or'ed key predicate
+;; (make-key-predicate '("<Control>j" "<Alt>k" "<Control>L"))
 (define make-key-predicate
   (lambda (sources)
     (cond
      ((list? sources)
-      (let ((predicates (mapcar make-single-key-predicate sources)))
+      (let ((predicates (map make-single-key-predicate sources)))
 	(lambda (key key-state)
 	  (apply proc-or
-		 (mapcar (lambda (predicate)
-			   (apply predicate (list key key-state)))
-			 predicates)))))
+		 (map (lambda (predicate)
+			(apply predicate (list key key-state)))
+		      predicates)))))
      (else
       (let ((source sources))
 	(make-single-key-predicate source))))))
@@ -260,7 +266,7 @@
   (lambda (key-strs)
     (cond
      ((list? key-strs)
-      (mapcar modify-key-strs-implicitly key-strs))
+      (map modify-key-strs-implicitly key-strs))
      ((string? key-strs)
       (let* ((key-str key-strs)
 	     (modified-key-str (string-append
@@ -271,22 +277,25 @@
       (let ((maybe-predicate key-strs))
 	maybe-predicate)))))
 
+;; Generates or'ed key predicate and bind it into
+;; toplevel-environment. Use define-key rather than calling this
+;; directly.
+;; (define-key-internal 'foo-key?  '("<Control>j" "<Alt>k" "<Control>L"))
 (define define-key-internal
   (lambda (key-predicate-sym key-strs)
     (let* ((modified-key-strs (modify-key-strs-implicitly key-strs))
-	   (predicate (make-key-predicate modified-key-strs))
-	   (toplevel-env ()))
+	   (predicate (make-key-predicate modified-key-strs)))
       (eval (list 'define key-predicate-sym predicate)
 	    toplevel-env))))
 
 ;(define set-key-binding
 ;  (lambda (im state flavor)
-;    nil
+;    #f
 ;    ))
 
 ;(define regist-key-binding 
 ;  (lambda (bind-name bind-table)
-;    nil
+;    #f
 ;))
 
 ;;
