@@ -1,8 +1,26 @@
 ;;
 ;;
 
+(define m17nlib-context-id
+  (lambda (mc)
+    (car (nthcdr 0 mc))))
+
+(define m17nlib-context-set-id!
+  (lambda (mc id)
+    (set-car! (nthcdr 0 mc) id)))
+
+(define m17nlib-context-new
+  (lambda (name)
+    (let ((c ()))
+      (set! c (copy-list '(())))
+      (m17nlib-context-set-id! c
+			       (m17nlib-lib-alloc-context name))
+      c)))
+
 (define m17nlib-init-handler
-  (lambda (id arg)))
+  (lambda (id arg)
+    (let* ((c (find-context id)))
+      (set-context-data! c (m17nlib-context-new arg)))))
 
 (define m17nlib-release-handler
   (lambda (id)))
@@ -11,7 +29,16 @@
   (lambda (id mode)))
 
 (define m17nlib-press-key-handler
-  (lambda (id key key-state)))
+  (lambda (id key key-state)
+    (let* ((c (find-context id))
+	   (mc (context-data c))
+	   (mid (m17nlib-context-id mc)))
+      (if
+       (m17nlib-lib-push-key mid key key-state)
+       (im-commit-raw id))
+      (let ((str (m17nlib-lib-get-str mid)))
+	(if str
+	    (im-commit id str))))))
 
 (define m17nlib-release-key-handler
   (lambda (id key key-state)))
@@ -51,6 +78,5 @@
 	  (m17nlib-register (+ i 1) nr-im))
 	())))
 
-; disabled for a while until m17n bugs are fixed
-;(m17nlib-lib-init)
-;(m17nlib-register 0 (m17nlib-lib-nr-input-methods))
+(m17nlib-lib-init)
+(m17nlib-register 0 (m17nlib-lib-nr-input-methods))
