@@ -45,32 +45,32 @@
 
 (require "japanese.scm")
 (require "generic-key.scm")
+(require "util.scm")
 
 ;; configs
-(define prime-use-candidate-window? #t)
-(define prime-candidate-op-count 1) ;;䥦ɥɽޤǤ˲󥹥ڡ򲡤ɬפ뤫
-(define prime-preedit-immediate-commit? #f)
-(define prime-always-show-window? #t)
-(define prime-mask-pending-preedit #f)
 (define prime-nr-candidate-max 10)
+(define prime-always-show-window? #t)
+(define prime-preedit-immediate-commit? #f)
+(define prime-mask-pending-preedit? #f)
 
 ;; config function
 (define prime-dont-use-numeral-key-to-select-cand
   (lambda ()
     (set! prime-cand-select-key?
 	  (lambda (key key-state)
-	    (and (numeral? key)
+	    (and (numeral-char? key)
 		 (control-key-mask key-state))))))
 
 ;; key
-(define-key prime-latin-key? '("<Control>l" "<Control>L" generic-off-key?))
-(define-key prime-wide-latin-key? nil)
-(define-key prime-begin-conv-key? " ")
+(define-key prime-latin-key? '("<Control>l" generic-off-key?))
+(define-key prime-wide-latin-key?  "<Control>L")
+(define-key prime-begin-conv-key? 'generic-begin-conv-key?)
 (define-key prime-on-key? '("<Control>j" "<Control>J" generic-on-key?))
 (define-key prime-commit-key? 'generic-commit-key?)
 (define-key prime-next-candidate-key? 'generic-next-candidate-key?)
 (define-key prime-prev-candidate-key? 'generic-prev-candidate-key?)
-(define-key prime-kana-toggle-key? '("<Control>q" "<Control>Q"))
+(define-key prime-next-page-key? 'generic-next-page-key?)
+(define-key prime-prev-page-key? 'generic-prev-page-key?)
 (define-key prime-cancel-key? 'generic-cancel-key?)
 (define-key prime-backspace-key? 'generic-backspace-key?)
 (define-key prime-delete-key? 'generic-delete-key?)
@@ -78,61 +78,55 @@
 (define-key prime-go-right-key? 'generic-go-right-key?)
 (define prime-cand-select-key?
   (lambda (key key-state)
-    (numeral? key)))
+    (numeral-char? key)))
 
 ;; access
 (define prime-context-raw-commit
  (lambda (ac)
-   (car (nthcdr 16 ac))))
+   (car (nthcdr 15 ac))))
 (define prime-context-set-raw-commit!
   (lambda (ac mode)
-    (set-car! (nthcdr 16 ac) mode)))
+    (set-car! (nthcdr 15 ac) mode)))
 (define prime-context-immediate-commit ;;Ūprime-preedit-immediate-commit?򥪥դˤ뤫ɤ
  (lambda (ac)
-   (car (nthcdr 15 ac))))
+   (car (nthcdr 14 ac))))
 (define prime-context-set-immediate-commit!
   (lambda (ac mode)
-    (set-car! (nthcdr 15 ac) mode)))
+    (set-car! (nthcdr 14 ac) mode)))
 (define prime-context-last-word ;;PRIMEPOBoxѸǤContext
  (lambda (ac)
-   (car (nthcdr 14 ac))))
+   (car (nthcdr 13 ac))))
 (define prime-context-set-last-word!
   (lambda (ac mode)
-    (set-car! (nthcdr 14 ac) mode)))
+    (set-car! (nthcdr 13 ac) mode)))
 (define prime-context-mode
  (lambda (ac)
-   (car (nthcdr 13 ac))))
+   (car (nthcdr 12 ac))))
 (define prime-context-set-mode!
   (lambda (ac mode)
-    (set-car! (nthcdr 13 ac) mode)))
+    (set-car! (nthcdr 12 ac) mode)))
 (define prime-context-left-string ;;κ¦ˤpreeditʸΥꥹ
  (lambda (ac)
-   (car (nthcdr 12 ac))))
+   (car (nthcdr 11 ac))))
 (define prime-context-set-left-string!
   (lambda (ac str)
-    (set-car! (nthcdr 12 ac) str)))
+    (set-car! (nthcdr 11 ac) str)))
 (define prime-context-right-string
  (lambda (ac)
-   (car (nthcdr 11 ac))))
+   (car (nthcdr 10 ac))))
 (define prime-context-set-right-string!
   (lambda (ac str)
-    (set-car! (nthcdr 11 ac) str)))
+    (set-car! (nthcdr 10 ac) str)))
 (define prime-context-candidates
   (lambda (c)
-   (car (nthcdr 10 c))))
+   (car (nthcdr 9 c))))
 (define prime-context-set-candidates!
   (lambda (c cnt)
-    (set-car! (nthcdr 10 c) cnt)))
+    (set-car! (nthcdr 9 c) cnt)))
 (define prime-context-candidate-window
-  (lambda (c)
-   (car (nthcdr 9 c))))
-(define prime-context-set-candidate-window!
-  (lambda (c cnt)
-    (set-car! (nthcdr 9 c) cnt)))
-(define prime-context-candidate-op-count
   (lambda (c)
    (car (nthcdr 8 c))))
-(define prime-context-set-candidate-op-count!
+(define prime-context-set-candidate-window!
   (lambda (c cnt)
     (set-car! (nthcdr 8 c) cnt)))
 (define prime-context-rk-context
@@ -165,23 +159,25 @@
 (define prime-context-learning-word
   (lambda (c)
     (car (nthcdr 2 c))))
-(define prime-context-kana-mode
-  (lambda (sc)
-    (car (nthcdr 1 sc))))
-(define prime-context-set-kana-mode!
-  (lambda (sc mode)
-    (set-car! (nthcdr 1 sc) mode)))
 (define prime-context-state
   (lambda (c)
     (car (nthcdr 0 c))))
 (define prime-context-set-state!
   (lambda (c s)
     (set-car! (nthcdr 0 c) s)))
-;; state kana head okuri tail candidates nth rk
 
-(define prime-mode-direct 0)
-(define prime-mode-hiragana 1)
-(define prime-mode-zenkaku 2)
+(define prime-mode-latin 0)
+(define prime-mode-wide-latin 1)
+(define prime-mode-hiragana 2)
+
+(define prime-send-command
+  (lambda (command)
+    (let ((result (prime-lib-send-command command)))
+      (let loop ((res result))
+	(if (string=? res "")
+	    (loop (prime-lib-send-command ""))
+	    res
+	    )))))
 
 (define prime-flush
   (lambda (sc)
@@ -200,39 +196,17 @@
   (lambda ()
     (let ((c 
 	   (copy-list 
-	    '(prime-state-latin #t () () () () 0 () () () () () () 0 "" #t #f))))
+	    '(prime-state-no-latin #t () () () () 0 () () () () () 0 "" #t #f))))
       (prime-context-set-rk-context! c
 				   (rk-context-new ja-rk-rule #t #f))
       (prime-flush c)
       c)))
 
-;; which is responsible to determine input mode?  -- YamaKen
-;;   - prime-context-mode
-;;   - prime-context-kana-mode and prime-context-wide-latin
-;;   - prime-context-state
-(define prime-context-kana-toggle
-  (lambda (sc)
-    (let ((kana (not (prime-context-kana-mode sc))))
-      (cond
-       (kana
-	(prime-context-set-state! sc 'prime-state-no-preedit)
-	(prime-context-set-mode! sc prime-mode-hiragana)
-	(prime-context-set-kana-mode! sc #t))
-       ((prime-context-wide-latin sc)
-	(prime-context-set-state! sc 'prime-state-direct)
-	(prime-context-set-mode! sc prime-mode-zenkaku)
-	(prime-context-set-kana-mode! sc #f))
-       ((not (prime-context-wide-latin sc))
-	(prime-context-set-state! sc 'prime-state-wide-latin)
-	(prime-context-set-mode! sc prime-mode-direct)
-	(prime-context-set-kana-mode! sc #f)))
-      (prime-context-set-kana-mode! sc kana))))
-
 (define prime-get-nth-candidate
   (lambda (sc n)
     (if (> n (prime-get-nr-candidates sc))
-	nil)
-    (car (cdr (car (nthcdr n (prime-context-candidates sc)))))))
+	#f
+	(car (cdr (car (nthcdr n (prime-context-candidates sc))))))))
 
 (define prime-get-nr-candidates
   (lambda (sc)
@@ -246,13 +220,86 @@
 
 (define prime-get-candidates! ;;⤦äȴؿ̾ɤˤ
   (lambda (sc preedit context)
-    (prime-lib-send-command (string-append "set_context\t"  context "\n"))
+    (if (not (string=? context ""))
+	(prime-engine-set-context context))
     (prime-context-set-candidates!
      sc
      (prime-parse-cands
-      (prime-lib-send-command (string-append "l\t"  preedit "\n"))))
+      (prime-send-command (string-append "lookup_compact\t" preedit "\n"))))
     ))
 
+(define prime-get-all-candidates! ;;⤦äȴؿ̾ɤˤ
+  (lambda (sc preedit context)
+    (if (not (string=? context ""))
+	(prime-engine-set-context context))
+    (prime-context-set-candidates!
+     sc
+     (prime-parse-cands
+      (prime-send-command
+       (string-append "lookup_compact_all\t" preedit "\n"))))
+    ))
+
+
+
+(define prime-get-label
+  (lambda (sc key)
+    (string-append
+     (string-list-concat (prime-context-left-string sc))
+     (charcode->string key))))
+
+;;;; By Hiroyuki Komatsu
+(define prime-engine-set-context
+  (lambda (string)
+    (prime-send-command (string-append "set_context\t" string "\n"))))
+
+;;;; By Hiroyuki Komatsu
+(define prime-engine-reset-context
+  (lambda ()
+    (prime-send-command "reset_context\n")))
+
+;;;; By Hiroyuki Komatsu
+(define prime-engine-get-label
+  (lambda (string)
+    (if (string=? string "")
+	""
+	(let ((label (prime-send-command (string-append "get_label\t" string "\n"))))
+	  (nth 1 (string-split label "\n"))))))
+
+;;;; By Hiroyuki Komatsu
+(define prime-engine-learn-word
+  (lambda (pron literal pos context suffix rest)
+    (prime-send-command
+     (string-append "learn_word\t" pron "\t" literal "\t" pos "\t"
+		    context "\t" suffix "\t" rest "\n"))))
+
+;;;; By Hiroyuki Komatsu
+(define prime-command-preedit-commit
+  (lambda (context key key-state)
+    (let ((sc (context-data context))
+	  (id (context-id   context)))
+      (if (prime-context-learning-word sc)
+	  (prime-commit-to-left-left sc (append
+					 (prime-context-right-string sc)
+					 (prime-context-left-string sc)))
+	  (let ((word-committed (prime-preedit-get-string-label sc)))
+	    (im-commit id word-committed)
+	    (prime-learn-word sc (list (list "basekey" word-committed)
+				       (list "base"    word-committed)))
+	    (prime-flush sc)
+	    (prime-update-mode id sc))))))
+
+;;;; By Hiroyuki Komatsu
+(define prime-preedit-get-string-label
+  (lambda (sc)
+    (prime-engine-get-label
+     (prime-preedit-get-string-raw sc))))
+
+;;;; By Hiroyuki Komatsu
+(define prime-preedit-get-string-raw
+  (lambda (sc)
+    (string-append (string-list-concat (prime-context-left-string sc))
+		   (string-list-concat (prime-context-right-string sc)))))
+
 (define prime-make-assoc-list
   (lambda (lst)
     (mapcar 
@@ -263,18 +310,16 @@
 (define prime-commit-raw
   (lambda (sc id)
     (im-commit-raw id)
+    (prime-engine-reset-context)
+    (prime-context-set-last-word! sc "")
     (prime-context-set-raw-commit! sc #t)))
 
 (define prime-commit-candidate
   (lambda (sc)
-    (let ((nth (prime-context-nth sc))
-	  (learning-word (prime-context-learning-word sc)))
-      (if learning-word
-	  (begin
-	    (set! assoc-list 
-		  (prime-make-assoc-list 
-		   (cdr (cdar (nthcdr nth (prime-context-candidates sc))))))
-	    (prime-learn-word sc assoc-list))))))
+    (let* ((nth (prime-context-nth sc))
+	   (assoc-list (prime-make-assoc-list 
+			(cdr (cdar (nthcdr nth (prime-context-candidates sc)))))))
+      (prime-learn-word sc assoc-list))))
 
 (define prime-learn-word
   (lambda (sc assoc-list)
@@ -285,12 +330,9 @@
 	  (suffix  (or (cadr (assoc "conjugation" assoc-list)) ""))
 	  (rest    (or (cadr (assoc "suffix"      assoc-list)) "")))
       
-      (prime-lib-send-command
-       (string-append "learn_word\t" key "\t" value "\t" part "\t" context "\t" suffix "\t" rest "\n"))
-      
-      (prime-context-set-last-word!
-       sc
-       (prime-get-current-candidate sc)))))
+      (prime-engine-learn-word key value part context suffix rest)
+      (prime-context-set-last-word! sc (string-append value suffix rest)))))
+;				    (prime-get-current-candidate sc)))))
 
 (define prime-parse-cands
   (lambda (cands-string)
@@ -303,7 +345,7 @@
   (lambda (sc id init-idx)
     (let ((res)
 	  (rkc (prime-context-rk-context sc)))
-      (prime-get-candidates!
+      (prime-get-all-candidates!
        sc
        (string-append (string-list-concat (prime-context-left-string sc))
 		      (string-list-concat (prime-context-right-string sc)))
@@ -341,7 +383,7 @@
 	      (right-right (prime-context-right-right-strings sc)))
 	  
 	  (if (and
-	       prime-mask-pending-preedit
+	       prime-mask-pending-preedit?
 	       left
 	       (usual-char? (string->charcode (car left))))
 	      (set! left (append (list "*") (cdr left))))
@@ -363,8 +405,8 @@
 		 id preedit-reverse
 		 (prime-get-current-candidate sc)))
 	      (if (prime-has-preedit? sc)
-		  (let ((hl (string-list-concat left))
-			(hr (string-list-concat (prime-context-right-string sc))))
+		  (let ((hl (prime-engine-get-label (string-list-concat left)))
+			(hr (string-list-concat right)))
 		    (if (string? hl)
 			(im-pushback-preedit
 			 id preedit-underline
@@ -399,16 +441,23 @@
     (let* ((rkc (prime-context-rk-context sc))
 	   (nth (prime-context-nth sc))
 	   (preedit (string-append (string-list-concat (prime-context-left-string sc))
-				   (string-list-concat (prime-context-right-string sc)))))
+				   (string-list-concat (prime-context-right-string sc))))
+	   (stat (prime-context-state sc)))
       (cond
        ((> nth 0)
 	(im-select-candidate id nth))
+       ((and (= nth 0)
+	     (= stat 'prime-state-converting))
+	(prime-get-all-candidates!
+	 sc
+	 preedit
+	 (prime-context-last-word sc))
+	(im-activate-candidate-selector
+	 id (prime-get-nr-candidates sc) prime-nr-candidate-max)
+	(im-select-candidate id nth))
        ((and
 	 (prime-has-preedit? sc)
 	 (not (prime-context-candidate-window sc))
-	 (or prime-always-show-window?
-	     (> (prime-context-candidate-op-count sc)
-		prime-candidate-op-count))
 	 (or (not (prime-context-immediate-commit sc))
 	     (not prime-preedit-immediate-commit?)))
 	(begin
@@ -416,24 +465,19 @@
 	   sc
 	   preedit
 	   (prime-context-last-word sc))
-	  (im-activate-candidate-selector
-	   id (prime-get-nr-candidates sc) prime-nr-candidate-max)
-	  (prime-context-set-candidate-window! sc #t)))
 	  
+	  (if (> (prime-get-nr-candidates sc) 0)
+	      (begin
+		(im-activate-candidate-selector
+		 id (prime-get-nr-candidates sc) prime-nr-candidate-max)
+		(prime-context-set-candidate-window! sc #t))
+	      (begin
+		(prime-context-set-candidate-window! sc #f)
+		(im-deactivate-candidate-selector id)))))
        ((not (prime-has-preedit? sc))
 	(begin
 	  (prime-context-set-candidate-window! sc #f)
-	  (im-deactivate-candidate-selector id)))
-	  
-       ((prime-context-candidate-window sc)
-	(begin
-	  (prime-get-candidates!
-	   sc
-	   preedit
-	   (prime-context-last-word sc))
-	  (if (= (prime-context-state sc)
-		 'prime-state-converting)
-	      (im-select-candidate id nth))))))))
+	  (im-deactivate-candidate-selector id)))))))
 
 (define prime-has-preedit?
   (lambda (sc)
@@ -471,7 +515,7 @@
 	   (id (context-id c))
 	   (key-str (charcode->string (to-lower-char key)))
 	   (rkc (prime-context-rk-context sc))
-	   (res nil)
+	   (res #f)
 	   (direct (ja-direct (charcode->string key)))
 	   (learning-word (prime-context-learning-word sc))
 	   (registered (string-append (string-list-concat (prime-context-left-left-strings sc))
@@ -479,21 +523,14 @@
       (cond
        ((prime-wide-latin-key? key key-state)
 	(begin
-	  (prime-context-set-mode! sc prime-mode-zenkaku)
-	  (prime-context-set-state! sc 'prime-state-wide-latin)
+	  (prime-context-set-mode! sc prime-mode-wide-latin)
 	  (prime-update-mode id sc)))
        
        ((prime-latin-key? key key-state)
 	(begin
-	  (prime-context-set-mode! sc prime-mode-direct)
-	  (prime-context-set-state! sc 'prime-state-latin)
+	  (prime-context-set-mode! sc prime-mode-latin)
 	  (prime-update-mode id sc)))
-       
-       ((prime-kana-toggle-key? key key-state)
-	(begin 
-	  (prime-context-kana-toggle sc)
-	  (prime-update-mode id sc)))
-       
+              
        ((prime-backspace-key? key key-state)
 	(if (not (rk-backspace rkc))
 	    (if (prime-context-left-left-strings sc)
@@ -519,9 +556,9 @@
        
        ((prime-commit-key? key key-state)
 	(if (and learning-word
-		 (not (string-equal? registered "")))
+		 (not (string=? registered "")))
 	    (begin
-	      (prime-lib-send-command
+	      (prime-send-command
 	       (string-append "learn_word\t" (string-list-concat learning-word) "\t" registered "\n"))
 	      (im-commit id registered)
 	      (prime-flush sc)
@@ -576,9 +613,9 @@
 	(if learning-word
 	    ()
 	    (begin
-	      (prime-flush sc)
 	      (prime-context-set-last-word! sc "")
-	      (prime-commit-raw sc id))))
+	      (prime-commit-raw sc id)
+	      (prime-flush sc))))
 
        (else
 	(begin
@@ -589,9 +626,6 @@
   (lambda (c key key-state)
     (let* ((sc (context-data c))
 	   (id (context-id c))
-	   (rkc (prime-context-rk-context sc))
-	   (stat (prime-context-state sc))
-	   (res ())
 	   (learning-word (prime-context-learning-word sc)))
       (cond
        ((or (prime-next-candidate-key? key key-state)
@@ -620,20 +654,12 @@
 	       (reverse
 		(cdr (reverse (prime-context-right-string sc))))))
 	  (prime-context-set-candidate-window! sc #f) ;FIXME:very dirty hack
-	  ))
+	  )) 
+
        ;; commit
        ((prime-commit-key? key key-state)
-	(if learning-word
-	    (prime-commit-to-left-left sc
-				       (append
-					(prime-context-right-string sc)
-					(prime-context-left-string sc)))
-	    (begin
-	      (im-commit id (string-append (string-list-concat (prime-context-left-string sc))
-					   (string-list-concat (prime-context-right-string sc))))
-	      (prime-flush sc)
-	      (prime-update-mode id sc))))
-       
+	(prime-command-preedit-commit c key key-state))
+
        ;; left
        ((prime-go-left-key? key key-state)
 	(begin
@@ -658,7 +684,7 @@
 		
        ((and (prime-cand-select-key? key key-state)
 	     (prime-context-immediate-commit sc))
-	(let* ((nth (- key 49))
+	(let* ((nth (number->candidate-index (numeral-char->number key)))
 	       (cand (prime-get-nth-candidate sc nth)))
 	  (if cand
 	      (if learning-word
@@ -683,19 +709,15 @@
 	  (prime-context-set-left-string!
 	   sc
 	   (string-to-list
-	    (nth 1 (string-split 
-		    (prime-lib-send-command 
-		     (string-append "get_label\t"
-				    (string-list-concat (prime-context-left-string sc))
-				    (charcode->string key) "\n")) "\n"))))
-	  
+	    (prime-get-label sc key)))
 	  (prime-context-set-candidate-window! sc #f) ;FIXME:very dirty hack
 	  (if (and
 	       prime-preedit-immediate-commit?
 	       (prime-context-immediate-commit sc))
 	      (begin
 		(im-commit id (string-list-concat (prime-context-left-string sc)))
-		(prime-flush sc)))))))))
+		(prime-flush sc)))))
+       ))))
 
 (define prime-proc-state-converting
   (lambda (c key key-state)
@@ -709,9 +731,7 @@
 	  (prime-context-set-nth! sc
 				  (+ 1 (prime-context-nth sc)))
 	  (if (prime-get-current-candidate sc)
-	      (prime-context-set-candidate-op-count!
-	       sc
-	       (+ 1 (prime-context-candidate-op-count sc)))
+	      #f
 	      (if learning-word
 		  (prime-context-set-nth! sc 0)
 		  (prime-word-learning-start! sc)))))
@@ -722,8 +742,12 @@
 	      (prime-context-set-nth! sc (- (prime-context-nth sc) 1))
 	      (prime-context-set-nth! sc (- (prime-get-nr-candidates sc) 1)))))
 
-       ((prime-cancel-key? key key-state)
-	(prime-flush sc))
+       ((or (prime-cancel-key? key key-state)
+	    (prime-backspace-key? key key-state))
+	(begin
+	  (prime-context-set-state! sc 'prime-state-preedit)
+	  (prime-context-set-nth! sc 0)
+	  (prime-context-set-candidate-window! sc #f)))
        
        ((prime-commit-key? key key-state)
 	(if learning-word
@@ -735,7 +759,7 @@
 	      (prime-update-mode id sc))))
 
        ((prime-cand-select-key? key key-state)
-	(let* ((nth (- key 49))
+	(let* ((nth (number->candidate-index (numeral-char->number key)))
 	       (cand (prime-get-nth-candidate sc nth)))
 	  (if cand
 	      (if learning-word
@@ -749,6 +773,9 @@
 		    (prime-flush sc)
 		    (prime-update-mode id sc))))))
 
+       ((symbol? key)
+	())
+
        (else
 	(begin
 	  (if learning-word
@@ -763,8 +790,7 @@
 		(prime-commit-candidate sc)
 		(prime-flush sc)
 		(prime-proc-input-no-preedit c key key-state)))
-	  (prime-update-preedit id sc))))
-      )))
+      ))))))
 
 
 (define prime-proc-mode-latin
@@ -801,9 +827,9 @@
 	   (fun)
 	   (res))
       (cond
-       ((= mode prime-mode-direct)
+       ((= mode prime-mode-latin)
 	(set! fun prime-proc-mode-latin))
-       ((= mode prime-mode-zenkaku)
+       ((= mode prime-mode-wide-latin)
 	(set! fun prime-proc-mode-wide-latin))
       ;((or (= mode 1) (= mode 2))  ;; what's intented? -- YamaKen
       ((= mode prime-mode-hiragana)
@@ -815,7 +841,7 @@
 	     (set! fun prime-proc-state-converting)))))
       (fun c key key-state)
       (prime-update-candidate-window sc (context-id c))
-      (prime-update-preedit (context-id c) sc)      
+      (prime-update-preedit (context-id c) sc)
       )))
 
 (define prime-init-handler
@@ -829,7 +855,7 @@
       (im-pushback-mode-list id "Ҥ餬")
       (im-pushback-mode-list id "ѱѿ")
       (im-update-mode-list id)
-      (im-update-mode id prime-mode-direct)
+      (im-update-mode id prime-mode-latin)
 
       (prime-update-prop-list id))))
 
@@ -846,7 +872,7 @@
 	   (sc (context-data c)))
       (if (or (control-char? key)
 	      (= (prime-context-mode sc)
-		 prime-mode-direct))
+		 prime-mode-latin))
 	  (im-commit-raw id)))))
 
 (define prime-reset-handler
@@ -859,17 +885,6 @@
 	   (sc (context-data c)))
       (prime-flush sc)
       (prime-context-set-mode! sc mode)
-      (cond
-       ((= mode prime-mode-direct)
-	(begin
-	  (prime-context-set-kana-mode! sc #f)
-	  (prime-context-set-wide-latin! sc #f)))
-       ((= mode prime-mode-hiragana)
-	(prime-context-set-kana-mode! sc #t))
-       ((= mode prime-mode-zenkaku)
-	(begin
-	  (prime-context-set-kana-mode! sc #f)
-	  (prime-context-set-wide-latin! sc #t))))
       (prime-update-preedit id sc)
       ())))
 
@@ -891,38 +906,30 @@
     (let* ((c (find-context id))
 	   (sc (context-data c)))
       (prime-flush sc)
-      (prime-update-preedit sc id)
+      (prime-update-preedit id sc)
       (cond
-       ((string-equal? message
-		       "prop_prime_hiragana")
-	(begin
-	  (prime-context-set-mode! sc prime-mode-hiragana)
-	  (prime-context-set-kana-mode! sc #t)))
-       ((string-equal? message
-		       "prop_prime_direct")
-	(begin
-	  (prime-context-set-mode! sc prime-mode-direct)
-	  (prime-context-set-kana-mode! sc #f)  ;; required? -- YamaKen
-	  (prime-context-set-wide-latin! sc #f)))
-       ((string-equal? message
-		       "prop_prime_zenkaku")
-	(begin
-	  (prime-context-set-mode! sc prime-mode-zenkaku)
-	  (prime-context-set-kana-mode! sc #f)  ;; required? -- YamaKen
-	  (prime-context-set-wide-latin! sc #t))))
+       ((string=? message
+		  "prop_prime_mode_hiragana")
+	(prime-context-set-mode! sc prime-mode-hiragana))
+       ((string=? message
+		  "prop_prime_mode_latin")
+	(prime-context-set-mode! sc prime-mode-latin))
+       ((string=? message
+		  "prop_prime_mode_wide_latin")
+	(prime-context-set-mode! sc prime-mode-wide-latin)))
       (prime-update-mode id sc)
       (prime-update-prop-label id sc))))
 
 (define prime-update-prop-label
   (lambda (id sc)
-    (let* ((state (prime-context-state sc))
+    (let* ((mode (prime-context-mode sc))
 	   (str ""))
       (cond
-       ((= state 'prime-state-latin)
+       ((= mode prime-mode-latin)
 	(set! str "P\tľ\n"))
-       ((= state 'prime-state-wide-latin)
+       ((= mode prime-mode-wide-latin)
 	(set! str "\tѱѿ\n"))
-       ((= state 'prime-state-no-preedit)
+       ((= mode prime-mode-hiragana)
 	(set! str "\tҤ餬\n")))
       (im-update-prop-label id str))))
 
@@ -930,20 +937,20 @@
   (lambda (id) 
     (let* ((c (find-context id))
 	   (sc (context-data c))
-	   (state (prime-context-state sc))
+	   (mode (prime-context-mode sc))
 	   (str ""))
       (cond
-       ((= state 'prime-state-latin)
+       ((= mode prime-mode-latin)
 	(set! str "P\tľ\n"))
-       ((= state 'prime-state-wide-latin)
+       ((= mode prime-mode-wide-latin)
 	(set! str "\tѱѿ\n"))
-       ((= state 'prime-state-no-preedit)
+       ((= mode prime-mode-hiragana)
 	(set! str "\tҤ餬\n")))
       (set! str 
 	    (string-append "branch\t" str
-			   "leaf\t\tҤ餬\tҤ餬ʥ⡼\tprop_prime_hiragana\n"
-			   "leaf\tP\tľ\tľϥ⡼\tprop_prime_latin\n"
-			   "leaf\t\tѱѿ\tѱѿ⡼\tprop_prime_wide_latin\n"))
+			   "leaf\t\tҤ餬\tҤ餬ʥ⡼\tprop_prime_mode_hiragana\n"
+			   "leaf\tP\tľ\tľϥ⡼\tprop_prime_mode_latin\n"
+			   "leaf\t\tѱѿ\tѱѿ⡼\tprop_prime_mode_wide_latin\n"))
       (im-update-prop-list id str)
       )))
 
@@ -951,9 +958,9 @@
  'prime
  "ja"
  "EUC-JP"
- nil
+ #f
  prime-init-handler
- nil
+ #f
  prime-mode-handler
  prime-press-key-handler
  prime-release-key-handler
