@@ -148,7 +148,7 @@
 	   (if (skk-cancel-key? key key-state)
 	       (begin
 		 (skk-back-to-kanji-state sc (skk-editor-id ec))
-		 (skk-context-set-child-context! sc nil)
+		 (skk-context-set-child-context! sc #f)
 		 #f)
 	       #t)
 	   )))))
