@@ -164,7 +164,7 @@
     (spell-context-set-tail! sc '())
     (spell-context-set-left-string! sc '())
     (spell-context-set-right-string! sc '())
-    (spell-context-set-nth! sc nil)
+    (spell-context-set-nth! sc #f)
     (spell-context-set-candidate-window! sc #f)))
 
 
@@ -212,12 +212,12 @@
 	(if (spell-context-kana-mode sc)
 	    (car res)
 	    (cdr res))
-	nil)))
+	#f)))
 
 (define spell-get-nth-candidate
   (lambda (sc n)
     (if (> n (spell-get-nr-candidates sc))
-	nil)
+	#f)
     (car (cdr (car (nthcdr n (spell-context-candidates sc)))))
     ))
 
@@ -373,7 +373,7 @@
 	   (id (context-id c))
 	   (key-str (charcode->string (to-lower-char key)))
 	   (rkc (spell-context-rk-context sc))
-	   (res nil)
+	   (res #f)
 	   (direct (ja-direct (charcode->string key)))
 	   (immediate-commit))
       (and
@@ -421,7 +421,7 @@
 	     #f)
 	   #t)
        (spell-proc-input-with-preedit c key key-state))
-      nil)))
+      #f)))
 
 (define spell-proc-input-with-preedit
   (lambda (c key key-state)
@@ -495,7 +495,7 @@
 		    (cons c (spell-context-left-string sc)))))
 	     #f)
 	   #t)
-       (if (numeral? key)
+       (if (numeral-char? key)
 	   (begin
 	     (spell-context-set-nth! sc (- key 49))
 	     (im-commit id (spell-get-nth-candidate sc (- key 49)))
@@ -529,7 +529,7 @@
 		     #f)
 		   #t)
 	       ))))
-      nil)))
+      #f)))
 
 (define spell-proc-state-converting
   (lambda (c key key-state)
@@ -665,13 +665,13 @@
  'spell
  "ja"
  "EUC-JP"
- nil
+ #f
  spell-init-handler
- nil
+ #f
  spell-mode-handler
  spell-press-key-handler
  spell-release-key-handler
  spell-reset-handler
  spell-get-candidate-handler
  spell-set-candidate-index-handler
- nil)
+ #f)
