@@ -1721,7 +1721,7 @@
       (set-context-data!
        c
        (generic-context-new tcode-rule #f))
-      ())))
+      #f)))
 
 (generic-register-im
  'tcode
