@@ -209,12 +209,24 @@
 (define (tutcode-key-press-handler id key key-state)
   (let* ((c (find-context id))
 	 (pc (context-data c)))
-    (if (and (number? key) (< key 32))
+    (if (control-char? key)
 	(im-commit-raw id)
 	(if (tutcode-context-on? pc)
 	    (tutcode-proc-on-mode id pc key key-state)
 	    (tutcode-proc-off-mode id pc key key-state)))))
 
+;;; Υ줿ȤνԤ
+;;; @param id IMID
+;;; @param key Ϥ줿
+;;; @param key-state ȥ륭ξ
+(define (tutcode-key-release-handler id key key-state)
+  (let* ((c (find-context id))
+	 (pc (context-data c)))
+    (if (or (control-char? key)
+	    (not (tutcode-context-on? pc)))
+	;; don't discard key release event for apps
+	(im-commit-raw id))))
+
 ;;; TUT-Code IMνԤ
 ;;; @param id IMID
 ;;; @param arg
@@ -304,7 +316,6 @@
 	       "leaf\t\t\tʥ⡼\tprop_tutcode_katakana\n"))
     (im-update-prop-list id str)))
 
-(define (tutcode-key-release-handler id key state))
 (define (tutcode-reset-handler id))
 (define (tutcode-get-candidate-handler id idx))
 (define (tutcode-set-candidate-index-handler id idx))
