@@ -200,7 +200,7 @@
       (set-context-data!
        c
        (generic-context-new viqr-rule #f))
-      ())))
+      #f)))
 
 (generic-register-im
  'viqr "vi" "UTF-8" viqr-init-handler)
