@@ -60,7 +60,7 @@ struct canna_context {
   int current_segment_num;
 };
 
-static struct canna_context *context_array;
+static struct canna_context *context_array = NULL;
 
 static int context_array_len;
 static int rk_initialized = -1;
@@ -224,7 +224,7 @@ create_context() {
 static LISP
 release_context(LISP id_)
 {
-  int id = get_c_long(id_);
+  int id = get_c_int(id_);
   struct canna_context *cc = get_canna_context(id);
 
   if(cc == NULL)
@@ -306,7 +306,7 @@ _update_segment (const int id, const int segment_num)
 static LISP
 begin_conversion(LISP id_, LISP str_)
 {
-  int id = get_c_long(id_);
+  int id = get_c_int(id_);
   char *str;
   int len, segment_num, mode;
   struct canna_context *cc = get_canna_context(id);
@@ -342,9 +342,9 @@ begin_conversion(LISP id_, LISP str_)
 static LISP
 get_nth_candidate(LISP id_, LISP seg_, LISP nth_)
 {
-  int id = get_c_long(id_);
-  int seg = get_c_long(seg_);
-  int nth = get_c_long(nth_);
+  int id = get_c_int(id_);
+  int seg = get_c_int(seg_);
+  int nth = get_c_int(nth_);
   struct canna_context *cc = get_canna_context(id);
   char buf[BUFSIZE];
   int len;
@@ -366,7 +366,7 @@ get_nth_candidate(LISP id_, LISP seg_, LISP nth_)
 static LISP
 get_nr_segments(LISP id_)
 {
-  int id = get_c_long(id_);
+  int id = get_c_int(id_);
   RkStat stat;
   struct canna_context *cc = get_canna_context(id);
 
@@ -382,8 +382,8 @@ get_nr_segments(LISP id_)
 static LISP
 get_nr_candidate(LISP id_, LISP nth_)
 {
-  int id = get_c_long(id_);
-  int nth = get_c_long(nth_);
+  int id = get_c_int(id_);
+  int nth = get_c_int(nth_);
   RkStat stat;
   struct canna_context *cc = get_canna_context(id);
 
@@ -404,9 +404,9 @@ get_nr_candidate(LISP id_, LISP nth_)
 static LISP
 resize_segment(LISP id_, LISP s_, LISP nth_)
 {
-  int id = get_c_long(id_);
-  int s = get_c_long(s_);
-  int nth = get_c_long(nth_);
+  int id = get_c_int(id_);
+  int s = get_c_int(s_);
+  int nth = get_c_int(nth_);
   struct canna_context *cc = get_canna_context(id);
 
   if(cc == NULL)
@@ -431,9 +431,9 @@ resize_segment(LISP id_, LISP s_, LISP nth_)
 static LISP
 commit_segment(LISP id_, LISP s_, LISP nth_)
 {
-  int id = get_c_long(id_);
-  int s = get_c_long(s_);
-  int nth = get_c_long(nth_);
+  int id = get_c_int(id_);
+  int s = get_c_int(s_);
+  int nth = get_c_int(nth_);
   struct canna_context *cc = get_canna_context(id);
 
   if(cc == NULL)
@@ -449,7 +449,7 @@ commit_segment(LISP id_, LISP s_, LISP nth_)
 static LISP
 reset_conversion(LISP id_)
 {
-  int id = get_c_long(id_);
+  int id = get_c_int(id_);
 
   _reset_conversion(id);
 }
@@ -474,16 +474,22 @@ uim_init_canna(void)
 void
 uim_quit_canna(void)
 {
-  if(cannaserver != NULL)
+  if(cannaserver != NULL) {
      free(cannaserver);
+     cannaserver = NULL;
+  }
 
   if(api.RkFinalize)
      api.RkFinalize();
 
-  if(api.lib)
+  if(api.lib) {
      dlclose(api.lib);
+     memset(&api, 0, sizeof(struct canna_api));
+  }
 
-  if(context_array != NULL)
+  if(context_array != NULL) {
      free(context_array);
+     context_array = NULL;
+  }
 }
 #endif /* HAVE_CANNA_RK_H */
