@@ -176,7 +176,7 @@ long siod_verbose_level;
 static LISP sym_t;
 /*  Added by Spencer Kimball for script-fu shit 6/3/97 */
 static FILE *siod_output;
-char *siod_lib;
+static const char *siod_lib;
 
 #define MAX_ERROR 1024
 static char siod_err_msg[MAX_ERROR];
@@ -220,7 +220,7 @@ static LISP *inums;
 static long inums_dim;
 static struct user_type_hooks *user_types;
 static long user_tc_next;
-struct gc_protected *protected_registers;
+static struct gc_protected *protected_registers;
 static jmp_buf save_regs_gc_mark;
 static double gc_rt;
 static long gc_cells_collected;
@@ -233,12 +233,66 @@ static long stack_size;
 
 static LISP lreadparen (struct gen_readio * f);
 static LISP lreadr (struct gen_readio *f);
-
+static LISP my_err(char *message, LISP obj);
+static LISP lprint (LISP exp, LISP);
 
 #define ENVLOOKUP_TRICK 1
 
 static long inside_err = 0;
 
+static char *
+try_get_c_string (LISP x)
+{
+  if TYPEP
+    (x, tc_symbol)
+      return (PNAME (x));
+  else if TYPEP
+    (x, tc_string)
+      return (x->storage_as.string.data);
+  else
+    return (NULL);
+}
+
+static LISP
+envlookup (LISP var, LISP env)
+{
+  LISP frame, al, fl, tmp;
+  for (frame = env; CONSP (frame); frame = CDR (frame))
+    {
+      tmp = CAR (frame);
+      if NCONSP
+	(tmp) my_err ("damaged frame", tmp);
+      for (fl = CAR (tmp), al = CDR (tmp); CONSP (fl); fl = CDR (fl), al = CDR (al))
+	{
+	  if NCONSP
+	    (al) my_err ("too few arguments", tmp);
+	  if EQ
+	    (CAR (fl), var) return (al);
+	}
+      /* suggested by a user. It works for reference (although conses)
+         but doesn't allow for set! to work properly... */
+#if (ENVLOOKUP_TRICK)
+      if (SYMBOLP (fl) && EQ (fl, var))
+	return (cons (al, NIL));
+#endif
+    }
+  if NNULLP
+    (frame) my_err ("damaged env", env);
+  return (NIL);
+}
+
+static LISP
+setvar (LISP var, LISP val, LISP env)
+{
+  LISP tmp;
+  if NSYMBOLP
+    (var) my_err ("wta(non-symbol) to setvar", var);
+  tmp = envlookup (var, env);
+  if NULLP
+    (tmp) return (VCELL (var) = val);
+  return (CAR (tmp) = val);
+}
+
 static LISP
 my_err (char *message, LISP x)
 {
@@ -307,46 +361,6 @@ my_err (char *message, LISP x)
   return (NIL);
 }
 
-static LISP
-envlookup (LISP var, LISP env)
-{
-  LISP frame, al, fl, tmp;
-  for (frame = env; CONSP (frame); frame = CDR (frame))
-    {
-      tmp = CAR (frame);
-      if NCONSP
-	(tmp) my_err ("damaged frame", tmp);
-      for (fl = CAR (tmp), al = CDR (tmp); CONSP (fl); fl = CDR (fl), al = CDR (al))
-	{
-	  if NCONSP
-	    (al) my_err ("too few arguments", tmp);
-	  if EQ
-	    (CAR (fl), var) return (al);
-	}
-      /* suggested by a user. It works for reference (although conses)
-         but doesn't allow for set! to work properly... */
-#if (ENVLOOKUP_TRICK)
-      if (SYMBOLP (fl) && EQ (fl, var))
-	return (cons (al, NIL));
-#endif
-    }
-  if NNULLP
-    (frame) my_err ("damaged env", env);
-  return (NIL);
-}
-
-static LISP
-setvar (LISP var, LISP val, LISP env)
-{
-  LISP tmp;
-  if NSYMBOLP
-    (var) my_err ("wta(non-symbol) to setvar", var);
-  tmp = envlookup (var, env);
-  if NULLP
-    (tmp) return (VCELL (var) = val);
-  return (CAR (tmp) = val);
-}
-
 static void
 init_slib_version (void)
 {
@@ -355,12 +369,6 @@ init_slib_version (void)
 	  NIL);
 }
 
-static char *
-siod_version (void)
-{
-  return ("3.2x 12-MAR-96 (with modifications for libuim)");
-}
-
 static struct user_type_hooks *
 get_user_type_hooks (long type)
 {
@@ -378,18 +386,6 @@ get_user_type_hooks (long type)
   return (NULL);
 }
 
-static void
-print_welcome (void)
-{
-  if (siod_verbose_level >= 2)
-    {
-      fprintf (siod_output, "Welcome to SIOD, Scheme In One Defun, Version %s\n",
-	       siod_version ());
-      fprintf (siod_output, "(C) Copyright 1988-1994 Paradigm Associates Inc. Help: (help)\n\n");
-      fflush (siod_output);
-    }
-}
-
 long
 no_interrupt (long n)
 {
@@ -400,8 +396,8 @@ no_interrupt (long n)
   return (x);
 }
 
-long
-get_c_long (LISP x)
+int
+get_c_int (LISP x)
 {
   if NINTNUMP
     (x) my_err ("not a number", x);
@@ -493,12 +489,13 @@ myruntime (void)
   GetProcessTimes (GetCurrentProcess (), &creation, &exit, &kernel, &user);
   return (kernel.dwLowDateTime * 1e7 + user.dwLowDateTime * 1e7);
 #endif
+  return 0.0;
 }
 
 static long
 repl (struct repl_hooks *h)
 {
-  LISP x, cw = 0;
+  LISP x;
   double rt, ct;
   while (1)
     {
@@ -584,12 +581,6 @@ fopen_c (char *name, char *how)
   return (fopen_cg (fopen, name, how));
 }
 
-static LISP
-fopen_l (LISP name, LISP how)
-{
-  return (fopen_c (get_c_string (name), NULLP (how) ? "r" : get_c_string (how)));
-}
-
 static void
 file_gc_free (LISP ptr)
 {
@@ -652,7 +643,7 @@ vload (char *fname, long cflag, long rflag)
 	  fnbuf = alloca(strlen(siod_lib) + strlen(fname) + 2);
 	  strcpy (fnbuf, siod_lib);
 	  strcat (fnbuf, "/");
-	  strcat (fnbuf, fname);
+     	  strcat (fnbuf, fname);
 	  if ((f = fopen (fnbuf, "r")))
 	    {
 	      fname = fnbuf;
@@ -822,7 +813,7 @@ flush_ws (struct gen_readio *f, char *eoferr)
 }
 
 LISP
-strcons (long length, char *data)
+strcons (long length, const char *data)
 {
   long flag;
   LISP s;
@@ -943,8 +934,7 @@ lreadstring (struct gen_readio * f)
 static LISP
 lreadsharp (struct gen_readio * f)
 {
-  LISP obj, l, result;
-  long j, n;
+  LISP obj;
   int c;
   c = GETC_FCN (f);
   switch (c)
@@ -1170,20 +1160,6 @@ set_fatal_exit_hook (void (*fcn) (void))
   fatal_exit_hook = fcn;
 }
 
-
-static char *
-try_get_c_string (LISP x)
-{
-  if TYPEP
-    (x, tc_symbol)
-      return (PNAME (x));
-  else if TYPEP
-    (x, tc_string)
-      return (x->storage_as.string.data);
-  else
-    return (NULL);
-}
-
 static LISP
 last (LISP l)
 {
@@ -1215,7 +1191,7 @@ stack_limit (LISP amount, LISP silent)
   if NNULLP
     (amount)
     {
-      stack_size = get_c_long (amount);
+      stack_size = get_c_int (amount);
       stack_limit_ptr = STACK_LIMIT (stack_start_ptr, stack_size);
     }
   if NULLP
@@ -1363,8 +1339,8 @@ static LISP
 ash (LISP value, LISP n)
 {
   long m, k;
-  m = get_c_long (value);
-  k = get_c_long (n);
+  m = get_c_int (value);
+  k = get_c_int (n);
   if (k > 0)
     m = m << k;
   else
@@ -1680,6 +1656,18 @@ symbol_value (LISP x, LISP env)
   return (tmp);
 }
 
+LISP
+symbol_to_string (LISP x, LISP env)
+{
+  LISP tmp;
+  if NSYMBOLP
+    (x) my_err ("not a symbol", x);
+  tmp = envlookup (x, env);
+  if NNULLP
+    (tmp) return (CAR (tmp));
+  tmp = strcons(-1, PNAME (x));
+  return (tmp);
+}
 
 
 char *
@@ -1692,20 +1680,28 @@ must_malloc (unsigned long size)
   return (tmp);
 }
 
+static int
+name_hash(const char *name)
+{
+  int hash = 0;
+  int c;
+  char *cname = (char *)name;
+  while ((c = *cname++)) {
+    hash = ((hash * 17) ^ c) % obarray_dim;
+  }
+  return hash;
+}
+
 static LISP
 gen_intern (const char *name)
 {
   LISP l, sym, sl;
   char *cname;
-  long hash = 0, n, c, flag;
+  long hash = 0, flag;
   flag = no_interrupt (1);
   if (obarray_dim > 1)
     {
-      hash = 0;
-      n = obarray_dim;
-      cname = (char *)name;
-      while ((c = *cname++))
-	hash = ((hash * 17) ^ c) % n;
+      hash = name_hash(name);
       sl = obarray[hash];
     }
   else
@@ -1728,6 +1724,48 @@ gen_intern (const char *name)
   return (sym);
 }
 
+static void
+unlink_from_sym_list(const char *name, LISP *lst)
+{
+  LISP cur, victim = NIL;
+  if (!strcmp(name, PNAME(CAR(*lst)))) {
+    victim = *lst;
+    *lst = CDR(*lst);
+  } else {
+    for (cur = *lst; cur && CDR(cur); cur = CDR(cur)) {
+      if (!strcmp(name, PNAME(CAR(CDR(cur))))) {
+	victim = CDR(cur);
+	CDR(cur) = CDR(CDR(cur));
+	break;
+      }
+    }
+  }
+  if (victim) {
+    free(PNAME(victim));
+  }
+}
+
+void
+do_undefine(const char *name)
+{
+  int hash;
+  unlink_from_sym_list(name, &oblistvar);
+  if (obarray_dim > 1) {
+      hash = name_hash(name);
+      unlink_from_sym_list(name, &obarray[hash]);
+  }
+}
+
+LISP
+undefine(LISP name)
+{
+  /* not yet tested */
+  if SYMBOLP(name) {
+    do_undefine(PNAME(name));
+  }
+  return NIL;
+}
+
 LISP
 rintern (const char *name)
 {
@@ -1740,7 +1778,7 @@ intern (LISP name)
   return (rintern (get_c_string (name)));
 }
 
-LISP
+static LISP
 subrcons (long type, char *name, SUBR_FUNC f)
 {
   LISP z;
@@ -1874,7 +1912,7 @@ string_prin1 (LISP ptr, struct gen_printio *f)
 static LISP
 string_equal (LISP a, LISP b)
 {
-  long j, len;
+  long len;
   len = a->storage_as.string.dim;
   if (len != b->storage_as.string.dim)
     return (NIL);
@@ -1963,7 +2001,6 @@ init_storage_1 (void)
   gc_protect_sym (&sym_t, "t");
   gc_protect_sym (&sym_f, "f");
   setvar (sym_t, sym_t, NIL);
-  setvar (rintern ("nil"), NIL, NIL);
   setvar (rintern ("let"), rintern ("let-internal-macro"), NIL);
   setvar (rintern ("let*"), rintern ("let*-macro"), NIL);
   setvar (rintern ("letrec"), rintern ("letrec-macro"), NIL);
@@ -2440,7 +2477,7 @@ gc_status (LISP args)
 static LISP
 gc_info (LISP arg)
 {
-  switch (get_c_long (arg))
+  switch (get_c_int (arg))
     {
     case 0:
       return NIL;
@@ -2762,7 +2799,12 @@ lapply (LISP fcn, LISP args)
 static LISP
 leval_setq (LISP args, LISP env)
 {
-  return (setvar (car (args), leval (car (cdr (args)), env), env));
+  if(symbol_boundp( car(args), env) == sym_t) {
+    return (setvar (car (args), leval (car (cdr (args)), env), env));
+  } else {
+    my_err ("unbound variable", car(args));
+    return NIL;
+  }
 }
 
 static LISP
@@ -2984,36 +3026,82 @@ reverse (LISP l)
 }
 
 static LISP
-let_macro (LISP form)
+split_to_name_and_value (LISP bindings)
 {
-  LISP p, fl, al, tmp;
+  LISP fl, al, binding;
   fl = NIL;
   al = NIL;
-  for (p = car (cdr (form)); NNULLP (p); p = cdr (p))
+  for (; NNULLP (bindings); bindings = cdr (bindings))
     {
-      tmp = car (p);
+      binding = car (bindings);
       if SYMBOLP
-	(tmp)
+	(binding)
 	{
-	  fl = cons (tmp, fl);
+	  fl = cons (binding, fl);
 	  al = cons (NIL, al);
 	}
       else
 	{
-	  fl = cons (car (tmp), fl);
-	  al = cons (car (cdr (tmp)), al);
+	  fl = cons (car (binding), fl);
+	  al = cons (cadr (binding), al);
 	}
     }
-  p = cdr (cdr (form));
+  return (cons (fl, al));
+}
+
+static LISP
+named_let_macro (LISP form)
+{
+  LISP name, fl, al, bindings, body;
+  
+  bindings = split_to_name_and_value (car (cddr (form)));
+  fl = car (bindings);
+  al = cdr (bindings);
+  
+  name = cadr (form);
+  body = cdr (cddr (form));
+  
+  setcar (form,
+          listn (3,
+                 rintern ("letrec"),               
+                 listn (1,
+                        listn (2,
+                               name,
+                               cons (sym_lambda, cons (reverse (fl), body)))),
+                 name));
+  setcdr (form, reverse (al));
+  return (form);
+}
+
+static LISP
+normal_let_macro (LISP form)
+{
+  LISP fl, al, bindings, body;
+
+  bindings = split_to_name_and_value (cadr (form));
+  fl = car (bindings);
+  al = cdr (bindings);
+  
+  body = cddr (form);
   if NULLP
-    (cdr (p)) p = car (p);
+    (cdr (body)) body = car (body);
   else
-    p = cons (sym_progn, p);
-  setcdr (form, cons (reverse (fl), cons (reverse (al), cons (p, NIL))));
+    body = cons (sym_progn, body);
+  setcdr (form, cons (reverse (fl), cons (reverse (al), cons (body, NIL))));
   setcar (form, rintern ("let-internal"));
   return (form);
 }
 
+static LISP
+let_macro (LISP form)
+{
+  if SYMBOLP
+    (cadr (form))
+      return (named_let_macro (form));
+  else
+    return (normal_let_macro (form));
+}
+
 static LISP
 leval_quote (LISP args, LISP env)
 {
@@ -3119,10 +3207,7 @@ lprin1g (LISP exp, struct gen_printio * f)
       break;
     case tc_intnum:
       n = (long) INTNM (exp);
-      if (((double) n) == INTNM (exp))
-	sprintf (tkbuffer, "%ld", n);
-      else
-	sprintf (tkbuffer, "%g", INTNM (exp));
+      sprintf (tkbuffer, "%ld", n);
       gput_st (f, tkbuffer);
       break;
     case tc_symbol:
@@ -3226,7 +3311,7 @@ print_to_string (LISP exp, LISP str, LISP nostart)
   return (str);
 }
 
-LISP
+static LISP
 lprint (LISP exp, LISP lf)
 {
   FILE *f = get_c_file (lf, siod_output);
@@ -3236,15 +3321,6 @@ lprint (LISP exp, LISP lf)
   return (NIL);
 }
 
-LISP
-flprint (LISP exp, LISP lf)
-{
-  FILE *f = get_c_file (lf, siod_output);
-  lprin1f (exp, f);
-  fput_st (f, "\n");
-  return (NIL);
-}
-
 LISP
 lprin1f (LISP exp, FILE * f)
 {
@@ -3510,7 +3586,7 @@ lungetc (LISP ii, LISP p)
   if NNULLP
     (ii)
     {
-      i = get_c_long (ii);
+      i = get_c_int (ii);
       f_ungetc (i, get_c_file (p, stdin));
     }
   return (NIL);
@@ -3590,7 +3666,7 @@ siod_verbose (LISP arg)
 {
   if NNULLP
     (arg)
-      siod_verbose_level = get_c_long (car (arg));
+      siod_verbose_level = get_c_int (car (arg));
   return (intcons (siod_verbose_level));
 }
 
@@ -3654,13 +3730,13 @@ lrand (LISP m)
     (m)
       return (intcons (res));
   else
-    return (intcons (res % get_c_long (m)));
+    return (intcons (res % get_c_int (m)));
 }
 
 LISP
 lsrand (LISP s)
 {
-  srand (get_c_long (s));
+  srand (get_c_int (s));
   return (NIL);
 }
 
@@ -3670,6 +3746,12 @@ siod_true_value (void)
   return (sym_t);
 }
 
+LISP
+siod_false_value (void)
+{
+  return (sym_f);
+}
+
 char *
 last_c_errmsg (int num)
 {
@@ -3710,25 +3792,25 @@ parser_read (LISP ignore)
 static LISP
 bitand (LISP a, LISP b)
 {
-  return (intcons (get_c_long (a) & get_c_long (b)));
+  return (intcons (get_c_int (a) & get_c_int (b)));
 }
 
 static LISP
 bitor (LISP a, LISP b)
 {
-  return (intcons (get_c_long (a) | get_c_long (b)));
+  return (intcons (get_c_int (a) | get_c_int (b)));
 }
 
 static LISP
 bitxor (LISP a, LISP b)
 {
-  return (intcons (get_c_long (a) ^ get_c_long (b)));
+  return (intcons (get_c_int (a) ^ get_c_int (b)));
 }
 
 static LISP
 bitnot (LISP a)
 {
-  return (intcons (~get_c_long (a)));
+  return (intcons (~get_c_int (a)));
 }
 
 
@@ -3766,7 +3848,7 @@ static LISP
 mapcar (LISP l)
 {
   LISP fcn = car (l);
-  switch (get_c_long (llength (l)))
+  switch (get_c_int (llength (l)))
     {
     case 2:
       return (mapcar1 (fcn, car (cdr (l))));
@@ -3781,7 +3863,7 @@ static LISP
 nth (LISP x, LISP li)
 {
   LISP l;
-  long j, n = get_c_long (x);
+  long j, n = get_c_int (x);
   for (j = 0, l = li; (j < n) && CONSP (l); ++j)
     l = CDR (l);
   if CONSP
@@ -4122,7 +4204,6 @@ init_subrs (void)
   setvar (rintern ("*parser_read.scm-loaded*"), sym_t, NIL);
   init_subr_0 ("eof-val", get_eof_val);
   init_subr_2 ("print", lprint);
-  init_subr_2 ("force-print", flprint);
   init_subr_2 ("prin1", lprin1);
   init_subr_3 ("print-to-string", print_to_string);
   init_subr_1 ("string-length", string_length);
@@ -4158,6 +4239,7 @@ init_subrs (void)
   init_subr_2 ("symbol-bound?", symbol_boundp);
   init_subr_2 ("symbol-value", symbol_value);
   init_subr_3 ("set-symbol-value!", setvar);
+  init_subr_2 ("symbol->string", symbol_to_string);
   init_fsubr ("the-environment", leval_tenv);
   init_subr_2 ("error", lerr);
   init_subr_0 ("quit", quit);
@@ -4216,6 +4298,7 @@ init_subrs (void)
   init_subr_1 ("typeof", ltypeof);
   init_subr_2 ("memv", memv);
   init_subr_2 ("member", member);
+  init_fsubr ("undefine", undefine);
   init_slib_version ();
 }
 
@@ -4280,7 +4363,6 @@ siod_init (int argc, char **argv, int warnflag, FILE *fp)
 {
   int k;
   char *ptr;
-  static int siod_lib_set = 0;
 
   siod_output = fp;
 
@@ -4336,16 +4418,6 @@ siod_init (int argc, char **argv, int warnflag, FILE *fp)
   fatal_exit_hook = NULL;
   stack_size = 50000;
 
-
-  if (!siod_lib_set)
-    {
-      if (getenv ("SIOD_LIB"))
-	{
-	  siod_lib = getenv ("SIOD_LIB");
-	  siod_lib_set = 1;
-	}
-    }
-
   /* parse arguments */
   for (k = 1; k < argc; ++k)
     {
