@@ -41,7 +41,7 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/stat.h>
-
+#include <unistd.h>
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
 
@@ -53,8 +53,6 @@
   (sizeof(*(su)) - sizeof((su)->sun_path) + strlen((su)->sun_path))
 #endif
 
-static int context_id = 0;
-
 static int nr_client_slots;
 static struct client {
   int fd;
@@ -77,6 +75,7 @@ init_serv_fd(char *path)
     return -1;
   }
 
+  bzero(&myhost, sizeof(myhost));
   myhost.sun_family = PF_UNIX;
   strcpy(myhost.sun_path, path);
 
@@ -88,7 +87,7 @@ init_serv_fd(char *path)
 
   chmod(path, S_IRUSR|S_IWUSR);
 
-  foo = listen(fd, SOMAXCONN);
+  foo = listen(fd, 5);
   if (foo == -1) {
     perror("failed in listen()");
     return -1;
@@ -140,7 +139,7 @@ parse_content(char *content, struct client *cl)
 int
 proc_func(struct client *cl)
 {
-  int i, rc;
+  int rc;
   char buf[BUFFER_SIZE];
 
   /* do read */
