@@ -76,28 +76,29 @@ uim_helper_fd(int fd, enum RorW rw)
     return FD_ISSET(fd, &fds) ? 1 : 0;
 }
 
-char*
+void
 uim_helper_send_message(int fd, const char *message)
 {
   int res;
 
   if (fd < 0)
-    return "";
+    return;
 
   if (!message)
-    return "";
+    return;
 
   /* readable and cannot read any character, means disconnected.
      so we should read here and proc such condition. */
 
   if (uim_helper_fd(fd, WRITE)) {
     int len = strlen(message);
-    char *buf = alloca(len + 2);
+    char *buf = malloc(len + 2);
     snprintf(buf, len + 2,"%s\n", message);
     res = write(fd, buf, len + 1);
+    free(buf);
   }
 
-  return "";
+  return;
 }
 
 char *
@@ -105,18 +106,20 @@ uim_helper_get_pathname(void)
 {
   char *path;
   char *login;
+  struct passwd *pw = NULL;
  
   login = getenv("LOGNAME");
   
-  if(!login) {
-    struct passwd *pw;
+  if (!login) {
     pw = getpwuid(getuid());
     login = strdup(pw->pw_name);
-    free(pw);
   }
 
   path = (char*)malloc(strlen(login)+ 20);
   sprintf(path, "/tmp/uimhelper-%s",login);
+  if (pw) {
+    free(login);
+  }
   return path;
 }
 
@@ -131,7 +134,7 @@ int uim_helpler_fd_writable(int fd)
 }
 
 
-int uim_helper_str_terminated(char *str)
+int uim_helper_str_terminated(const char *str)
 {
   if(!str)
     return 0;
