@@ -217,9 +217,6 @@ uim_ipc_send_command(int *pid, FILE **read_handler,
     strcat(tmp, buf);
   }
  
-  if (strlen(tmp) == 0)
-    goto err;
-
   return tmp;
 
  err:
