@@ -1,3 +1,35 @@
+/*
+
+  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+
+  All rights reserved.
+
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
+
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+  3. Neither the name of authors nor the names of its contributors
+     may be used to endorse or promote products derived from this software
+     without specific prior written permission.
+
+  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  SUCH DAMAGE.
+*/
+
 // classes for preedit draw
 
 #include <X11/Xlib.h>
@@ -717,10 +749,12 @@ bool ConvdispOv::check_atr()
 	m_atr->line_space = 14;
     }
     if (!m_atr->has_atr(ICA_Foreground)) {
-	m_atr->foreground_pixel = BlackPixel(XimServer::gDpy, DefaultScreen(XimServer::gDpy));
+	m_atr->foreground_pixel = BlackPixel(XimServer::gDpy,
+					     DefaultScreen(XimServer::gDpy));
     }
     if (!m_atr->has_atr(ICA_Background)) {
-	m_atr->background_pixel = WhitePixel(XimServer::gDpy, DefaultScreen(XimServer::gDpy));
+	m_atr->background_pixel = WhitePixel(XimServer::gDpy,
+					     DefaultScreen(XimServer::gDpy));
     }
     return true;
 }
@@ -741,6 +775,7 @@ void ConvdispOv::make_ce_array()
     }
 }
 
+// Dirty,,, 
 void ConvdispOv::layoutCharEnt()
 {
     int i;
@@ -767,9 +802,14 @@ void ConvdispOv::layoutCharEnt()
 	XmbTextExtents(m_atr->font_set, buf, l, &ink, &logical);
 	m_ce[i].width = logical.width;
 	m_ce[i].height = logical.height;
+	int right_limit = m_atr->area.width;
+	if (m_atr->has_atr(ICA_Area)) {
+	    right_limit += m_atr->area.x;
+	}
 	if (m_ce[i].width + x >
-	    m_atr->area.width /*+ m_atr->area.x*/) {
-	    x = 0;/*m_atr->area.x;*/
+	    right_limit) {
+	    // goto next line
+	    x = m_atr->area.x;
 	    y += m_atr->line_space;
 	}
 	m_ce[i].x = x;
